#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
skeppa
------------
:copyright: (c) 2016 Martin Sandström
:license: MIT, see LICENSE for more details.
"""


__title__ = "skeppa"
__version__ = "1.4.0"
__build__ = 140
__license__ = "MIT"
__copyright__ = "Copyright 2016 Martin Sandström"
